import 'dart:io';

import 'package:flutter/material.dart';

import 'package:flutter_i18n/flutter_i18n.dart';
import 'package:flutter_platform_alert/flutter_platform_alert.dart';

import 'package:vivanta_connect_flutter/data/brand.dart';
import 'package:vivanta_connect_flutter/data/client.dart';
import 'package:vivanta_connect_flutter/helpers/helpers.dart';
import 'package:vivanta_connect_flutter/helpers/vivanta_sync.dart';
import 'package:vivanta_connect_flutter/services/apple_health.dart';
import 'package:vivanta_connect_flutter/services/database.dart';
import 'package:vivanta_connect_flutter/services/google_health_connect.dart';
import 'package:vivanta_connect_flutter/services/http.dart';
import 'package:vivanta_connect_flutter/services/preferences.dart';
import 'package:vivanta_connect_flutter/styles/colors.dart';
import 'package:vivanta_connect_flutter/styles/fonts.dart';
import 'package:vivanta_connect_flutter/views/loading.dart';
import 'package:vivanta_connect_flutter/views/webview.dart';
import 'package:vivanta_connect_flutter/widgets/brand_item.dart';

const appleHealthId = 6;
const googleHealthConnectId = 8;

class Brands extends StatefulWidget {
  const Brands({Key? key, this.client}) : super(key: key);

  final Client? client;

  @override
  State<Brands> createState() => _BrandsState();
}

class _BrandsState extends State<Brands> {
  bool _loading = false;
  bool showSearch = false;
  String _apiKey = '';
  String _customerId = '';
  String _externalUserId = '';
  List<Brand> _webBrands = [];
  List<HealthPermission> _appleHealthPermissions = [];
  List<HealthPermission> _googleHealthPermissions = [];
  TextEditingController searchController = TextEditingController();
  FocusNode searchFocus = FocusNode();
  final http = Http();
  final storage = Preferences();

  @override
  void initState() {
    _getBrands();
    searchController.addListener(() => setState(() {}));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return _loading
        ? const Loading()
        : Scaffold(
            backgroundColor: VivantaColors.background,
            appBar: AppBar(
              backgroundColor: VivantaColors.background,
              elevation: 0,
              title: showSearch
                  ? TextFormField(
                      decoration: InputDecoration(
                        hintText:
                            FlutterI18n.translate(context, "brands.search"),
                        hintStyle: VivantaFonts.title.copyWith(height: 1),
                        labelStyle: VivantaFonts.title.copyWith(height: 1),
                        contentPadding: const EdgeInsets.all(4),
                        isDense: true,
                      ),
                      style: VivantaFonts.title,
                      controller: searchController,
                      focusNode: searchFocus,
                    )
                  : Text(
                      FlutterI18n.translate(context, "brands.title"),
                      style: VivantaFonts.title,
                    ),
              actions: [
                Padding(
                  padding: const EdgeInsets.only(right: 16),
                  child: GestureDetector(
                    onTap: () {
                      setState(() {
                        showSearch = !showSearch;
                        if (showSearch) {
                          searchFocus.requestFocus();
                        } else {
                          searchController.text = '';
                          searchFocus.unfocus();
                        }
                      });
                    },
                    child: Icon(
                      showSearch ? Icons.close : Icons.search,
                      color: VivantaColors.text,
                      size: 24,
                    ),
                  ),
                ),
              ],
              leading: GestureDetector(
                onTap: () {
                  Navigator.of(context).maybePop();
                },
                child: const Icon(
                  Icons.arrow_back_ios,
                  color: VivantaColors.text,
                  size: 24,
                ),
              ),
            ),
            body: GridView.count(
              crossAxisCount: 2,
              children: [
                ..._getBrandItems(),
              ],
            ),
          );
  }

  Future<void> _getBrands() async {
    setState(() {
      _loading = true;
    });
    _apiKey = await storage.getApiKey();
    _customerId = await storage.getCustomerId();
    _externalUserId = await storage.getExternalUserId();
    final token = await storage.getToken();
    _webBrands = await http.getBrands(token);
    setState(() {
      _loading = false;
    });
  }

  Future<void> _requestAppleHealthPermission() async {
    final database = Database();
    final preferences = Preferences();
    await database.init();
    final appleHealth = AppleHealth();
    final authorized =
        await appleHealth.requestAuthorization(_appleHealthPermissions);

    if (authorized) {
      await preferences.setAppleHealthAccessGranted(authorized);
      await preferences.setAppleHealthAccessGrantedDate(DateTime.now());

      await FlutterPlatformAlert.showAlert(
        windowTitle: _getTranslation("apple.title_success"),
        text: _getTranslation("apple.text_success"),
        alertStyle: AlertButtonStyle.ok,
        iconStyle: IconStyle.information,
      );

      VivantaSync(
        apiKey: _apiKey,
        customerId: _customerId,
        externalUserId: _externalUserId,
      ).executeAppleHealth(database, preferences, http);

      return;
    } else {
      await FlutterPlatformAlert.showAlert(
        windowTitle: _getTranslation("apple.title_error"),
        text: _getTranslation("apple.text_error"),
        alertStyle: AlertButtonStyle.ok,
        iconStyle: IconStyle.error,
      );
      return;
    }
  }

  Future<void> _requestGoogleHealthConnectPermission() async {
    final database = Database();
    final preferences = Preferences();
    await database.init();
    final healthConnect = GoogleHealthConnect();

    final isHealthConnectInstalled = await healthConnect.isInstalled();

    if (!isHealthConnectInstalled) {
      final button = await FlutterPlatformAlert.showAlert(
        windowTitle: _getTranslation("google_health.title_not_installed"),
        text: _getTranslation("google_health.text_not_installed"),
        alertStyle: AlertButtonStyle.okCancel,
        iconStyle: IconStyle.information,
      );

      if (button == AlertButton.okButton) {
        healthConnect.install();
      }
      return;
    }

    final isGoogleFitInstalled = await healthConnect.isGoogleFitInstalled();

    if (!isGoogleFitInstalled) {
      final button = await FlutterPlatformAlert.showAlert(
        windowTitle:
            _getTranslation("google_health.title_not_installed_google_fit"),
        text: _getTranslation("google_health.text_not_installed_google_fit"),
        alertStyle: AlertButtonStyle.okCancel,
        iconStyle: IconStyle.information,
      );

      if (button == AlertButton.okButton) {
        healthConnect.installGoogleFit();
      }
      return;
    }

    final granted = await preferences.getGoogleHealthConnectAccessGranted();
    final authorized =
        await healthConnect.requestAuthorization(_googleHealthPermissions);

    if (authorized) {
      if (!granted) {
        await preferences.setGoogleHealthConnectAccessGranted(authorized);
        await preferences
            .setGoogleHealthConnectAccessGrantedDate(DateTime.now());
      }
      final button = await FlutterPlatformAlert.showAlert(
        windowTitle: _getTranslation("google_health.title_success"),
        text: _getTranslation("google_health.text_success"),
        alertStyle: AlertButtonStyle.yesNo,
        iconStyle: IconStyle.information,
      );

      if (button == AlertButton.yesButton) {
        healthConnect.open();
      }

      VivantaSync(
        apiKey: _apiKey,
        customerId: _customerId,
        externalUserId: _externalUserId,
      ).executeGoogleHealthConnect(database, preferences, http);

      return;
    }

    final button = await FlutterPlatformAlert.showAlert(
      windowTitle: _getTranslation("google_health.title_error"),
      text: _getTranslation("google_health.text_error"),
      alertStyle: AlertButtonStyle.yesNo,
      iconStyle: IconStyle.error,
    );

    if (button == AlertButton.yesButton) {
      healthConnect.open();
    }

    return;
  }

  String _getTranslation(String key) {
    return FlutterI18n.translate(context, key);
  }

  List<BrandItem> _getBrandItems() {
    _appleHealthPermissions = getAppleHealthPermissionFromClient(widget.client);
    _googleHealthPermissions =
        getGoogleHealthPermissionFromClient(widget.client);

    final hasAppleHealth = _webBrands.any((element) => element.id == 6);
    final hasGoogleHealth = _webBrands.any((element) => element.id == 8);

    if (!Platform.isIOS) {
      _removeAppleHealth();
    } else if (_appleHealthPermissions.isEmpty) {
      _removeAppleHealth();
    }

    if (!Platform.isAndroid) {
      _removeGoogleHealthConnect();
    } else if (_googleHealthPermissions.isEmpty) {
      _removeGoogleHealthConnect();
    }

    if (!hasAppleHealth &&
        _appleHealthPermissions.isNotEmpty &&
        Platform.isIOS) {
      _addAppleHealth();
    }

    if (!hasGoogleHealth &&
        _googleHealthPermissions.isNotEmpty &&
        Platform.isAndroid) {
      _addGoogleHealthConnect();
    }

    final filteredBrands = _webBrands
        .where((element) => element.name!
            .toLowerCase()
            .contains(searchController.text.toLowerCase()))
        .toList();

    filteredBrands.sort(
      (a, b) => a.name!.toLowerCase().compareTo(b.name!.toLowerCase()),
    );

    return filteredBrands
        .map<BrandItem>(
          (brand) => BrandItem(
            brand: brand,
            onTap: (brand) {
              if (brand.id == 6) {
                _requestAppleHealthPermission();
                return;
              }
              if (brand.id == 8) {
                _requestGoogleHealthConnectPermission();
                return;
              }
              final url =
                  'https://connect.vivanta.io/native/$_apiKey/$_customerId/$_externalUserId/${brand.id}';
              Navigator.of(context).push(
                MaterialPageRoute(
                  builder: (context) => Webview(
                    brand: brand,
                    url: url,
                  ),
                ),
              );
            },
          ),
        )
        .toList();
  }

  void _removeAppleHealth() {
    _webBrands.removeWhere((element) => element.id == appleHealthId);
  }

  void _addAppleHealth() {
    final apple = Brand(
      id: appleHealthId,
      name: 'Apple',
      image: 'https://vivanta-doc.s3.amazonaws.com/brand-apple-square.png',
    );
    _webBrands.add(apple);
  }

  void _removeGoogleHealthConnect() {
    _webBrands.removeWhere((element) => element.id == googleHealthConnectId);
  }

  void _addGoogleHealthConnect() {
    final google = Brand(
      id: googleHealthConnectId,
      name: 'Google Health Connect',
      image:
          'https://vivanta-doc.s3.amazonaws.com/brand-google-health-square.png',
    );
    _webBrands.add(google);
  }

  @override
  void dispose() {
    searchController.dispose();
    searchFocus.dispose();
    super.dispose();
  }
}
